import peasy.*;
import treemap.*; //Tree Map library
PeasyCam cam;

int startYear = 2007;
int endYear = 2016;
Table table;
Table deweyTable;
int numRows, numColumns;
ArrayList<Category> categories = new ArrayList<Category>();
ArrayList<ArrayList<Category>> deweyData;
HashMap<Integer, String> deweyMap;
String[] deweyNames;
float[] position= new float[3];
ArrayList<double[]> Axis;
//Setting

final int NUM_LAYERS = 12 * 10;   //Number of Layers;
                             
final int LAYER_DIST = 70;  //Distance between each layer in virtual space 

final int TRANS = 100; // Transparacny of layers

//Variables for drawing rects
ArrayList<ArrayList<PVector>> pointsOne = new ArrayList<ArrayList<PVector>>();
ArrayList<ArrayList<PVector>> pointsTwo = new ArrayList<ArrayList<PVector>>();
ArrayList<ArrayList<PVector>> pointsThree = new ArrayList<ArrayList<PVector>>();

boolean showAxis = true;

void setup(){
  size(1440, 900, P3D);
  cam = new PeasyCam(this, 500);

  //Added on Jan 30th by Rodger Luo
  //float  cameraZ = (height/2.0) / tan(PI*60.0/360.0);
  //perspective(PI/3.0, width/height, 0.001, cameraZ*10.0); // set near clip to 0.001 to go inside data
  // get value and deweyClasses name hashmap 
  deweyTable = loadTable("data/deweyClasses.csv");
  deweyMap = new HashMap<Integer,String>();
  for(int i=0; i<deweyTable.getRowCount(); i++){
    if(deweyTable.getInt(i,0) == 0 || deweyTable.getInt(i,0) % 100 == 0){
      deweyMap.put(deweyTable.getInt(i,0),deweyTable.getString(i,1)); 
    }
  }
  // put names array
  deweyNames = new String[deweyMap.size()];
  for(int i =0; i<deweyMap.size(); i++){
    deweyNames[i] = deweyMap.get(i*100);
  }
  // initialize data arraylist
  deweyData = new ArrayList<ArrayList<Category>>();
  for(int i=0; i< deweyMap.size(); i++){
    deweyData.add(new ArrayList<Category>());
  }

  table = loadTable("data/output2.csv", "header");
  numRows = table.getRowCount();
  for(int i=0; i<numRows; i++){
    int year = table.getInt(i,0);
    int deweyNumber = table.getInt(i,2);
    String month = table.getString(i,1);
    String categoryName = deweyMap.get(deweyNumber);
    int count = table.getInt(i,3);
    int index = deweyNumber / 100;
    deweyData.get(index).add(new Category(deweyNumber, categoryName, year, month, count));
  }
  for (int i=0; i<deweyData.size(); i++) { 
    pointsOne.add(new ArrayList<PVector>());
    pointsTwo.add(new ArrayList<PVector>());
    pointsThree.add(new ArrayList<PVector>());
  }
  for (int i=0; i<deweyData.size(); i++) {
    for(int j=0; j < deweyData.get(i).size(); j++){
      float x1 = deweyData.get(i).get(j).x1;
      float y1 = deweyData.get(i).get(j).y1;
      float x2 = deweyData.get(i).get(j).x2;
      float y2 = deweyData.get(i).get(j).y2;
      
      PVector pointOne = new PVector(0, 0, LAYER_DIST*(dateParser(deweyData.get(i).get(j).year,deweyData.get(i).get(j).month) - NUM_LAYERS/2));
      pointsOne.get(i).add(pointOne);
      
      PVector pointTwo = new PVector(x1, y1,  LAYER_DIST*(dateParser(deweyData.get(i).get(j).year,deweyData.get(i).get(j).month) - NUM_LAYERS/2));
      pointsTwo.get(i).add(pointTwo);

      PVector pointThree = new PVector(x2, y2,  LAYER_DIST*(dateParser(deweyData.get(i).get(j).year,deweyData.get(i).get(j).month) - NUM_LAYERS/2));
      pointsThree.get(i).add(pointThree);
        
    }
  }
  Axis = new ArrayList<double[]>();
  for( int i=0; i < deweyData.size(); i++){
    double xAxis = (int)(Math.floor(Math.random()*2)) == 1 ? Math.random()*250 + 300 : -Math.random()*550 - 500;
    double yAxis = (int)(Math.floor(Math.random()*2)) == 1 ? Math.random()*250 + 300 : -Math.random()*250 - 300;  
    double zAxis = (int)(Math.floor(Math.random()*2)) == 1 ? Math.random()*700*6 : -Math.random()*70*6; 
    Axis.add(new double[] {xAxis, yAxis, zAxis});
  }
}

void draw(){
  background(0);
  colorMode(HSB); 
  //draw bottom meshes
  pushMatrix();
  translate(0, 0);
  for (int i=0; i<deweyData.size(); i++) {
    for(int j=0; j < deweyData.get(i).size()-1; j++){
      float x1_1 = pointsOne.get(i).get(j).x;
      float y1_1 = pointsOne.get(i).get(j).y;
      float z1_1 = pointsOne.get(i).get(j).z;
      
      float x2_1 = pointsTwo.get(i).get(j).x;
      float y2_1 = pointsTwo.get(i).get(j).y;
      float z2_1 = pointsTwo.get(i).get(j).z;
      
      float x1_2 = pointsOne.get(i).get(j+1).x;
      float y1_2 = pointsOne.get(i).get(j+1).y;
      float z1_2 = pointsOne.get(i).get(j+1).z;
      
      float x2_2 = pointsTwo.get(i).get(j+1).x;
      float y2_2 = pointsTwo.get(i).get(j+1).y;
      float z2_2 = pointsTwo.get(i).get(j+1).z;
      
      int lineColor = deweyData.get(i).get(j).deweyNumber;
      beginShape();  
      noStroke();
      fill(map(lineColor, 0, 990, 0, 255), 255/2, 255/2, TRANS);
      vertex(x1_1, y1_1, z1_1);
      vertex(x2_1, y2_1, z2_1);
      vertex(x2_2, y2_2, z2_2);
      vertex(x1_2, y1_2, z1_2);
      endShape();        
    }
  }  
  popMatrix();
  
  //draw top meshes
  pushMatrix();
  translate(0, 0);
  for (int i=0; i<deweyData.size(); i++) {
    for(int j=0; j < deweyData.get(i).size()-1; j++){
      float x1_1 = pointsOne.get(i).get(j).x;
      float y1_1 = pointsOne.get(i).get(j).y;
      float z1_1 = pointsOne.get(i).get(j).z;
      
      float x2_1 = pointsThree.get(i).get(j).x;
      float y2_1 = pointsThree.get(i).get(j).y;
      float z2_1 = pointsThree.get(i).get(j).z;
      
      float x1_2 = pointsOne.get(i).get(j+1).x;
      float y1_2 = pointsOne.get(i).get(j+1).y;
      float z1_2 = pointsOne.get(i).get(j+1).z;
      
      float x2_2 = pointsThree.get(i).get(j+1).x;
      float y2_2 = pointsThree.get(i).get(j+1).y;
      float z2_2 = pointsThree.get(i).get(j+1).z;
      
      int lineColor = deweyData.get(i).get(j).deweyNumber;
      beginShape();  
      noStroke();
      fill(map(lineColor, 0, 990, 0, 255), 255/2, 255/2, TRANS);
      vertex(x1_1, y1_1, z1_1);
      vertex(x2_1, y2_1, z2_1);
      vertex(x2_2, y2_2, z2_2);
      vertex(x1_2, y1_2, z1_2);
      endShape();        
    }
  }  
  popMatrix();
  
  //draw outter meshes
  pushMatrix();
  translate(0, 0);
  for (int i=0; i<deweyData.size(); i++) {
    for(int j=0; j < deweyData.get(i).size()-1; j++){
      float x1_1 = pointsTwo.get(i).get(j).x;
      float y1_1 = pointsTwo.get(i).get(j).y;
      float z1_1 = pointsTwo.get(i).get(j).z;
      
      float x2_1 = pointsThree.get(i).get(j).x;
      float y2_1 = pointsThree.get(i).get(j).y;
      float z2_1 = pointsThree.get(i).get(j).z;
      
      float x1_2 = pointsTwo.get(i).get(j+1).x;
      float y1_2 = pointsTwo.get(i).get(j+1).y;
      float z1_2 = pointsTwo.get(i).get(j+1).z;
      
      float x2_2 = pointsThree.get(i).get(j+1).x;
      float y2_2 = pointsThree.get(i).get(j+1).y;
      float z2_2 = pointsThree.get(i).get(j+1).z;
      
      int lineColor = deweyData.get(i).get(j).deweyNumber;
      beginShape();  
      noStroke();
      fill(map(lineColor, 0, 990, 0, 255), 255/2, 255/2, TRANS);
      vertex(x1_1, y1_1, z1_1);
      vertex(x2_1, y2_1, z2_1);
      vertex(x2_2, y2_2, z2_2);
      vertex(x1_2, y1_2, z1_2);
      endShape();        
    }
  }  
  popMatrix();
  // draw data layer
  for (int i=0; i<deweyData.size(); i++) {
    for(int j=0; j < deweyData.get(i).size(); j++){
        pushMatrix();
        translate(0, 0, LAYER_DIST*(dateParser(deweyData.get(i).get(j).year,deweyData.get(i).get(j).month) - NUM_LAYERS/2));
        deweyData.get(i).get(j).draw();
        popMatrix();
    }
  }
  
  PFont myHelvetica = createFont("Helvetica",80);
  // draw years
  for ( int i =0; i <= NUM_LAYERS; i++){
    if(i == 0 || i % 12 == 0){
      pushMatrix();
      translate(0, 0, LAYER_DIST*(i - NUM_LAYERS/2));
      stroke(map(i/12, 0, 12, 0, 255), 255/5, 255, TRANS*2);
      strokeWeight(3);
      noFill();
      ellipse(0, 0, 800, 800);
      int year = 2006 + i /12;
      int radius = 430;
      fill(255);
      textSize(100);
      textFont(myHelvetica);
      if (year < 2013){
        text(String.valueOf(year),radius*(float)Math.cos(PI/ 5 * i /12)-240,140+radius*(float)Math.sin(PI/5*i/12));
      } else {
        text(String.valueOf(year),radius*(float)Math.cos(PI/ 5 * i /12),radius*(float)Math.sin(PI/5*i/12));
      }
      popMatrix();
    }
  }
  // draws legend

  for ( int i =0; i < deweyNames.length; i++){
   pushMatrix();
   translate((float)Axis.get(i)[0],(float)Axis.get(i)[1],(float)Axis.get(i)[2]);
   strokeWeight(2);
   fill(map(i*100, 0, 900, 0, 255), 255/2, 255, TRANS*2);
   textSize(300);
   textFont(myHelvetica);
   text(deweyNames[i],0,0);
   popMatrix();
  }
}